<?php
// No direct access
defined( '_JEXEC' ) or die;

use DiDom\Element;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Uri\Uri;
use DiDom\Document;

require_once __DIR__.DIRECTORY_SEPARATOR.'libs'.DIRECTORY_SEPARATOR.'main.php';

/**
 *
 * @package     Joomla.Plugin
 * @subpackage  System.Honeysuckle
 * @since       2.5+
 * @author		Vladimir Egorov
 */
class plgSystemHoneysuckle extends JPlugin
{
	/**
	 * Class Constructor
	 * @param object $subject
	 * @param array $config
	 */

	public $mainclass;
	public $componentParams;
    public $checkFromSpamList;
    public $isSpam;
    public $Check;

	public function __construct( & $subject, $config )
	{
	    $this->mainclass = new HoneysuckleMain($config['params']);

		parent::__construct( $subject, $config );
		$this->loadLanguage();
	}

	public function onAfterInitialise(){

        $app = Factory::getApplication();
        $input = $app->getInput();

        $action = $input->get('action', '');
        $key = $input->get('key', '');

        if ($action == 'setsessionfield' && $key) {
            $this->setSessionNameField($key) ;
        }

        $this->componentParams = ComponentHelper::getParams('com_honeysuckle');

        if ($app->isClient('administrator')) {
            return;
        }

        if ( $this->componentParams->get('access_only_for_ip', '') != '') {
            $this->checkAccess( $this->componentParams->get('access_only_for_ip', ''));
        }

	    $ip = $this->mainclass->getUserIpAddr();

	    $this->checkFromSpamList = $this->mainclass->checkIpFromSpamList($ip);

        $this->isSpam = 0;
        $this->Check = 1;

        if ($this->checkFromSpamList) {
            if ($this->checkFromSpamList->published == 0) {
                $this->isSpam = 1;
            }

            if ($this->checkFromSpamList->published == 1) {
                $this->Check = 0;
                $this->isSpam = 0;
            }
        }

        if ($this->isSpam == 1) {
            $this->mainclass->SpamPage();
        }

        $this->addSpamopronaLog($ip);

    }

    private function setSessionNameField($key) {
        if ($key != $this->params->get('key', '')) {
            die();
        }

        $extension_id = PluginHelper::getPlugin('system','honeysuckle')->id;

        $db = Factory::getContainer()->get('DatabaseDriver');
        $query = $db
            ->getQuery(true)
            ->select('*')
            ->from($db->quoteName('#__extensions'))
            ->where($db->quoteName('extension_id') . " = " . $extension_id);

        $db->setQuery($query);
        $plugin = $db->loadObject();
        $params = (object)json_decode($plugin->params);
        $newkey = $this->generateRandomString(12);
        $params->session_name_field = $newkey;

        $plugin->params = json_encode($params);

        $db->updateObject('#__extensions', $plugin, 'extension_id');
    }

    private function generateRandomString($length = 10) {
        $characters = 'abcdefghijklmnopqrstuvwxyz';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[random_int(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function checkAccess($ips) {

        $ips_arr = explode(',', $ips);
        $current_ip = $this->mainclass->getUserIpAddr();
        $yes = 0;

        foreach ($ips_arr as $item) {
            if (trim($item) == $current_ip) {
                $yes = 1;
            }
        }

        if (!$yes) {
            die('Access Denied');
        }
    }


    public function addSpamopronaLog($ip) {
        $app = Factory::getApplication();
        $user = $app->getIdentity();

	    $object = new stdClass();
	    $requests = $_REQUEST;
        $object->ip = $ip;

	    if (count($requests) > 0) {

            $chck_for_url = false;

            if ( $this->componentParams->get('check_for_stop_words', 0) == 1) {

                $chck_for_url = $this->mainclass->checkRequestText(print_r($_REQUEST, true));

            }

            if ($chck_for_url && $this->Check) {
                $object->published = 0;
                $this->mainclass->addToSpam($ip, print_r($_REQUEST, true));
                $this->setLog($object, $ip);
                $this->mainclass->SpamPage();
            } else {
                $object->published = 1;
            }


            $check = true;

            if (!$user->guest ) {
                $check = false;
            }

            $input = $app->input;
            $option = $input->get('option', '');
            $denied_components = $this->componentParams->get('denied_components', array());

            if (count($denied_components)) {
                if (in_array($option, $denied_components)) {
                    $check = false;
                }
            }

            $field_actions = $this->params->get('session_name_field', '', 'RAW');

            if ($option == 'com_j_sms_registration' && $check) {
                $this->checkJSmsRegistration($requests, $field_actions, $ip);
            }

            if ($option == 'com_backcall' && $check) {
                $this->checkJSmsRegistration($requests, $field_actions, $ip);
            }

            if ($check && $this->Check) {


                $check_field = $input->get($field_actions, 0);

                if ((!$check_field && isset($requests[$field_actions])) ) {
                    $object->published = 0;
                    $this->mainclass->addToSpam($ip, print_r($_REQUEST, true));
                    $this->setLog($object, $ip);
                    $this->mainclass->SpamPage();
                }
            }


            $object->published =1;
            $this->setLog($object, $ip);

        }

    }

    public function checkJSmsRegistration($requests, $field_actions, $ip) {

        if (!isset($requests[$field_actions])) {
            $object = new stdClass();
            $object->published = 0;
            $this->mainclass->addToSpam($ip, print_r($_REQUEST, true));
            $this->setLog($object, $ip);
            $this->mainclass->SpamPage();
        }
    }

    public function checkForComponents($field_actions) {

	    $app = Factory::getApplication();
	    $input = $app->input;

	    $tasks = array();
	    $tasks[] = 'contact.submit';
	    $tasks[] = 'user.login';
	    $tasks[] = 'saveUser';

	    $task = $input->get('task', '', 'RAW');
	    $check = $input->get($field_actions, 0);

	    if (in_array($task, $tasks) && !$check) {
	        return true;
        }

	    return false;

    }

    public function setLog($object, $ip) {

        $uri = JUri::getInstance();
        $object->url = $uri->toString();

        $object->log = print_r($_REQUEST, true);

        $object->created = date('Y-m-d H:i:s');

        $object->ip = $ip;

        Factory::getDbo()->insertObject('#__honeysuckle_logs', $object);

        $this->deleteLogs();
    }

    public function deleteLogs() {

	    $duration = $this->componentParams->get('storage_time_of_logs', 60);

        $time = new DateTime();
        $time->sub(new DateInterval('PT' . $duration . 'M'));

        $newtime = $time->format('Y-m-d H:i:s');

        $db = JFactory::getDbo();

        $query = $db->getQuery(true);

        $conditions = array(
            $db->quoteName('created') . ' < ' . $db->quote($newtime)
        );

        $query->delete($db->quoteName('#__honeysuckle_logs'));
        $query->where($conditions);

        $db->setQuery($query);

        $db->execute();

    }

    public function isPluginPage() {
        $app = Factory::getApplication();
        $input = $app->getInput();

        $option = $input->get('option', '');
        $view = $input->get('view', '');
        $layout = $input->get('layout', '');
        $extension_id = $input->get('extension_id', '');

        if ($option == 'com_plugins' && $view == 'plugin' && $layout == 'edit' && $extension_id == PluginHelper::getPlugin('system','honeysuckle')->id) {
            return true;
        }

        return false;
    }

    private function adminPlugin() {
        $app = Factory::getApplication();

        if ($this->isPluginPage() && $this->params->get('key', '')) {
            $html = '<script>';
            $html .= 'var PLG_SPAM_OBORONA_CRON_URL = "'.Text::_('PLG_SPAM_OBORONA_CRON_URL').'";';
            $html .= 'var cron_key ="'.JUri::root().'index.php?action=setsessionfield&key='.$this->params->get('key', '').'";';
            $html .= '</script>';
            $html .= '<script src="'.JUri::root().'plugins/system/honeysuckle/libs/jquery.js"></script>';
            $html .= '<script src="'.JUri::root().'plugins/system/honeysuckle/libs/admin.js"></script>';
        }

        $sHtml = $app->getBody();

        $sHtml = str_replace('</body>', $html.'</body>', $sHtml);

        $app->setBody($sHtml);

    }



	function onAfterRender(){


        $app = Factory::getApplication();
        $user = $app->getIdentity();

        if ($app->isClient('administrator')) {
            $this->adminPlugin();
            return;
        }


        $field_name = $this->params->get('session_name_field', 'fldsessname');

        $input_html = '<input type="hidden" value="0" class="sitogon_click_counter" name = "'.$field_name.'" />';

        $sHtml = $app->getBody();

        preg_match_all('/<form[^>]*?>(.*?)<\/form>/si', $sHtml, $forms);

        if ($user->guest) {

            foreach ($forms[0] as $form) {
                $dom = new DomDocument();
                $dom->loadHTML($form);
                $set = 0;

                $selects = $dom->getElementsByTagName("select");

                if ($selects->length) {
                    $set = 1;
                }

                $texareas= $dom->getElementsByTagName("textarea");

                if ($texareas->length) {
                    $set = 1;
                }

                $inputs = $dom->getElementsByTagName("input");

                if ($inputs->length) {
                    foreach ($inputs as $input) {
                        $type = $input->getAttribute('type');
                        $name = $input->getAttribute('name');

                        $denieds = array();
                        $denieds[] = 'quantity[]';

                        if ($type !== 'hidden' && !in_array($name, $denieds )) {
                            $set = 1;
                        }
                    }
                }


                if ($set) {

                    $old_form = $form;
                    $new_form = str_replace('</form>', $input_html.'</form>', $form);

                    $sHtml = str_replace($old_form, $new_form, $sHtml);
                }


            }
        }



        $form = '<form id="sitogonbottomform">';
        $form .= $input_html;
        $form .= '</form>';

        $dop_script = '<script src="'.Uri::root().'plugins/system/honeysuckle/libs/script.js"></script>';

        $sHtml = str_replace('</body>',$form.$dop_script.'</body>', $sHtml );

        $app->setBody($sHtml);


    }


}
